/**
 *  $Id: Iterator.java,v 1.1 2002/11/06 14:08:21 serge Exp $
 *
 * Transmorpher
 *
 * Copyright (C) 2001-2002 Fluxmedia and INRIA Rhne-Alpes.
 *
 * http://www.fluxmedia.fr - http://transmorpher.inrialpes.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package fr.fluxmedia.transmorpher.graph;

// imported fr.fluxmedia.transmorpher classes

import fr.fluxmedia.transmorpher.utils.Writer;
import fr.fluxmedia.transmorpher.engine.TIterator;
import fr.fluxmedia.transmorpher.engine.TLoop;
import fr.fluxmedia.transmorpher.engine.TProcess;
import fr.fluxmedia.transmorpher.engine.TProcessComposite;
import fr.fluxmedia.transmorpher.engine.TProcessFactory;
import fr.fluxmedia.transmorpher.utils.Parameters;

//imported java classes

import java.util.Enumeration;
import java.io.IOException;

/**
 *  Transmorpher graph iterator implementation The iterator class introduces the
 *  use of a variable which will be iterated over a particular structure. It
 *  must contain a name, a type and a number of other parameters that will
 *  specify its behavior.
 *
 *@author    Jerome.Euzenat@inrialpes.fr
 *@since     jdk 1.3 / SAX 2.0
 */
public class Iterator {

	/**  Iterator name. */
	String name = null;

	/**  The iterator parameters. */
	Parameters parameters = null;

	/** Iterator type */
	String type = null;

	/**  The owner . It must be a Repeat */
	Repeat repeat = null;

	/**
	 *  Creates an instance of iterator.
	 *
	 *@param  n  the name of the iterator.
	 *@param  t  the type of the iterator.
	 *@param  r  the owner of the iterator.
	 */
	public Iterator(String n, String t, Repeat r) {
		parameters = new Parameters();
		name = n;
		type = t;
		repeat = r;
	}

	/**
	 *  Prints the XML expression of the call. This is a default printer that can
	 *  be overloaded
	 */
	public void generateXML() {
		System.out.print("        <iterate name=\"" + name + "\" type=\"" + type + "\"");
		if (parameters.isEmpty()) {
			System.out.println("/>");
		} else {
			System.out.println(">");
			parameters.generateXMLCalls();
			System.out.println("        </iterate>");
		}
	}

	/**
	 *  Returns the name.
	 *
	 *@return    the name of the iterator.
	 */
	public String getName() {
		return name;
	}

	/**
	 *  Returns the owner (repeat) of the iterator.
	 *
	 *@return    the owner.
	 */
	public Repeat getRepeat() {
		return repeat;
	}

	/**
	 *  Sets the type of iterator.
	 *
	 *@param  t  the type.
	 */
	public void setType(String t) {
		type = t;
	}

	/**
	 *  Returns iterator type.
	 *
	 *@return    the type.
	 */
	public String getType() {
		return type;
	}

	/**
	 *  Sets parameters.
	 *
	 *@param  param  The new parameters value
	 */
	public void setParameters(Parameters param) {
		parameters = param;
	}

	/**
	 *  Returns parameters.
	 *
	 *@return    parameters.
	 */
	public Parameters getParameters() {
		return parameters;
	}

	/**
	 *  Creates a new TIterator corresponding to the iterator and adds it to the
	 *  current composite process (this must be a TLoop because iterators cannot be
	 *  used inside others components)
	 *
	 *@param  process  the current TLoop.
	 *@param  factory  used for creating the TIterator.
	 */
	public void createIterator(TProcessComposite process, TProcessFactory factory) {

		TIterator iterator = TProcessFactory.newIterator(name, type, getParameters());
		if (process instanceof TLoop) {
			((TLoop) process).addIterator(iterator);
		} else {
			System.out.println("Can not add an Iterator to this process");
		}
	}

	/**
	 *  Generates java code for this component.
	 *
	 *@param  file  The file to write the code in
	 */
	public void generateJavaCode(Writer file) throws IOException {
	    file.writeln();
	    file.writeln(4, "//------------------ Generation of Iterator " + name + " -------------------");
	    file.writeln(4, "name=\"" + name + "\";");
	    file.writeln(4, "type=\"" + type + "\";");
	    generateParameters(file);
	    
	    file.writeln(4, "iterator = tmProcessFactory.newIterator(name,type,tmParameters);");
	    file.writeln(4, "if (tmCurrentCompositeProcess instanceof TLoop) {");
	    file.writeln(8, "((TLoop) tmCurrentCompositeProcess).addIterator(iterator);");
	    file.writeln(4, "}");
	}

	/**
	 *  Generates java code for the parameters of this iterator.
	 *
	 *@param  file  The file to write the code in.
	 */
	public void generateParameters(Writer file) throws IOException {
	    file.writeln();
	    file.writeln(4, "tmParameters = new Parameters(); ");
	    for (Enumeration e = getParameters().getNames(); e.hasMoreElements(); ) {
		String key = (String) e.nextElement();
		String val = (String) getParameters().getParameter(key);
		file.writeln(4, "tmParameters.setParameter(\"" + key + "\",\"" + val + "\" );");
	    }//end for
	    file.writeln();
	}//end proc
}

